package getpde.Seisan;

import java.text.SimpleDateFormat;
import java.util.Locale;

public class Line1 extends SFileLine {

    public final int TYPE = 1;
    public long date;
    public boolean fixTime = false;
    public String locationModelIndicator = " ";
    public String distanceIndicator = " ";
    public String eventID = " ";
    public double latitude;
    public double longitude;
    public double depth = 0;
    public String depthIndicator = " ";
    public String locatingIndicator = " ";
    public String hypoReportingAgency = "   ";
    public int NSta = 0;
    public double rms = 0;
    public double Magnitude1 = 0;
    public String Magnitude1Type = " ";
    public String Magnitude1ReportingAgency = "   ";

    /*
     * Method to produce a line type 1 from a USGS header
     */
    public static Line1 fromUSGS(String line) throws Exception {
        Line1 newLine = new Line1();
        //Event Type
        newLine.distanceIndicator = "D"; //For global events
        //Agency
        newLine.hypoReportingAgency = "PDE";
        //Date
        newLine.date = new SimpleDateFormat("yy/MM/dd HH:mm:ss").parse(line.substring(0, 17)).getTime();
        //Latitude
        newLine.latitude = ((line.charAt(24) == 'N') ? 1 : -1) * Double.valueOf(line.substring(19, 24).trim());
        //Longitude
        newLine.longitude = ((line.charAt(32) == 'E') ? 1 : -1) * Double.valueOf(line.substring(26, 32).trim());
        //Depth
        newLine.depth = Double.valueOf(line.substring(34, 39).trim());
        //Magnitude
        newLine.Magnitude1 = Double.valueOf(line.substring(40, 43).trim());
        newLine.Magnitude1ReportingAgency = "PDE";

        //TODO: M and Q

        return newLine;
    }

    /*
     * toString will be used to generate a string with the line 1 type
     */
    @Override
    public String toString() {
        String output = " ";
        output += String.format(Locale.US,
                "%s %4.1f%s%s%s%7.3f%8.3f%5.1f%s%s%s%3d%4.1f%4.1f%s%s                %d",
                new SimpleDateFormat("yyyy MMdd HHmm").format(date),
                Double.valueOf(new SimpleDateFormat("ss.SS").format(date)), //seconds
                this.locationModelIndicator,
                this.distanceIndicator,
                this.eventID,
                this.latitude,
                this.longitude,
                this.depth,
                this.depthIndicator,
                this.locatingIndicator,
                this.hypoReportingAgency,
                this.NSta,
                this.rms,
                this.Magnitude1,
                this.Magnitude1Type,
                this.Magnitude1ReportingAgency,
                this.TYPE);
        return output;
    }
}
